Class SPGame extends xDeathmatch config;

var bool bEndGame;
var bool bTransition;

Struct Saved_PRI
{
        Var Int LightSuitPower;
        Var Int BlinderPower;
        Var Int StealthPower;
	Var Int Credits;
};

Struct Saved_Weapon
{
        Var String WeaponClass;
        Var Int Primary_Ammo;
        Var Int Secondary_Ammo;
};

Struct Variable_Info
{
        Var String Variable_Name;
        Var String Variable_Value;
};

Struct Saved_Inventory
{
        Var String Inventory_Class;
        Var Array<Variable_Info> Inventory_Variables;
};

Struct Saved_Player_Details
{
        Var Array<Saved_Weapon> Player_Weapons;
        Var Array<Saved_Inventory> Player_Inventory;
        Var Saved_PRI Player_PRI;
        
        Var String Player_Hash;       
};

Var Config Array<Saved_Player_Details> Player_Info;

Function ResetPlayerInfo()
{
        Player_Info.Length = 0;
	SaveConfig();
}

function Bot SpawnBot(optional string botName)
{
    	local Bot B;

    	return B;
}

function bool AddBot(optional string botName)
{
	return false;
}

function NavigationPoint FindPlayerStart( Controller Player, optional byte InTeam, optional string incomingName )
{
    local NavigationPoint N, BestStart;
    local Teleporter Tel;
    local float BestRating, NewRating;
    local byte Team;

    // always pick StartSpot at start of match
    if ( (Player != None) && (Player.StartSpot != None) && (Level.NetMode == NM_Standalone)
        && (bWaitingToStartMatch || ((Player.PlayerReplicationInfo != None) && Player.PlayerReplicationInfo.bWaitingPlayer))  )
    {
        return Player.StartSpot;
    }

    if ( GameRulesModifiers != None )
    {
        N = GameRulesModifiers.FindPlayerStart(Player,InTeam,incomingName);
        if ( N != None )
            return N;
    }

    // if incoming start is specified, then just use it
    if( incomingName!="" )
        foreach AllActors( class 'Teleporter', Tel )
            if( string(Tel.Tag)~=incomingName )
                return Tel;

    // use InTeam if player doesn't have a team yet
    if ( (Player != None) && (Player.PlayerReplicationInfo != None) )
    {
        if ( Player.PlayerReplicationInfo.Team != None )
            Team = Player.PlayerReplicationInfo.Team.TeamIndex;
        else
            Team = InTeam;
    }
    else
        Team = InTeam;

    for ( N=Level.NavigationPointList; N!=None; N=N.NextNavigationPoint )
    {
        NewRating = RatePlayerStart(N,Team,Player);
        if ( NewRating > BestRating )
        {
            BestRating = NewRating;
            BestStart = N;
        }
    }

    if ( SPPlayerStart(BestStart) == None )
    {
        log("Warning - PATHS NOT DEFINED or NO PLAYERSTART with positive rating");
		BestRating = -100000000;
        ForEach AllActors( class 'NavigationPoint', N )
        {
            NewRating = RatePlayerStart(N,0,Player);
            if ( InventorySpot(N) != None )
				NewRating -= 50;
			NewRating += 20 * FRand();
            if ( NewRating > BestRating )
            {
                BestRating = NewRating;
                BestStart = N;
            }
        }
    }

    return BestStart;
}

function float RatePlayerStart(NavigationPoint N, byte Team, Controller Player)
{
    local PlayerStart P;
    local float Score;
    //local Controller OtherPlayer;

    P = SPPlayerStart(N);

    if ( (P == None) || !P.bEnabled || P.PhysicsVolume.bWaterVolume )
        return -99999999999999;

    else if ( P.bEnabled )
		Score = 99999999999999999999999999;

    return FMax(Score, 5);
}

event InitGame(string Options, out string Error)
{
	Super.InitGame(Options, Error);
	SetGameSpeed(0.9);
}

simulated function Timer()
{
	super.Timer();

	if (bEndGame)
	{
		EndGame(None, "TimeLimit");
	}
}

auto State PendingMatch
{
	function RestartPlayer(Controller aPlayer)
	{
		if ( CountDown <= 0 )
			Super.RestartPlayer(aPlayer);
	}

    function bool AddBot(optional string botName)
    {
        if ( Level.NetMode == NM_Standalone )
            InitialBots++;
        if ( botName != "" )
			PreLoadNamedBot(botName);
		else
			PreLoadBot();
        return true;
    }

    	function Timer()
    	{
		Global.Timer();
	}

    function beginstate()
    {
		bWaitingToStartMatch = false;
        StartupStage = 0;
	//StartMatch();
    }

Begin:
	StartMatch();
}

function int ReduceDamage(int Damage, pawn injured, pawn instigatedBy, vector HitLocation, out vector Momentum, class<DamageType> DamageType)
{
	if (Monster(Injured) != None)
	{
		if (Monster(Injured).SameSpeciesAs(InstigatedBy))
			return 0;
		//return Damage;
	}

	if (instigatedBy.IsA('Pawn') && !instigatedBy.IsA('Monster') && injured.IsA('Pawn') && !injured.IsA('Monster'))
	{
		return 0;
		//return damage;
	}

	else
		return Super.ReduceDamage(Damage,injured,instigatedBy,HitLocation,Momentum,DamageType);
}

function AddGameSpecificInventory(Pawn P)
{
	local SPLevelInfo I;

    if (bAllowTrans)
	Log("Allowing translocators, but we're not going to spawn these in this gametype, duh. If a player or server admin sees this message, disable translocators from the in-game menu!");

	if (!P.IsA('SPMonster'))
	{
		foreach AllActors(class'SPLevelInfo', I)
		{
			if (I != None)
			{
				I.LoadMeUp(P);
			}
		}

	}
}

Function DumpPlayerInfo(PlayerController C)
{
        Local Int i, k;
        Local Inventory Inv;

	i = -1;
        
	For (i = 0; i < Player_Info.Length; i++)
	{
		If (Player_Info[i].Player_Hash == C.GetPlayerIDHash())
		{
			Player_Info.Remove(i, 1);
		}
	}

	If (i == -1)
	{
		i = Player_Info.Length;
	}
        else
        {
		i = Player_Info.Length;
        }

	Player_Info.Insert(i, 1);

	Player_Info[i].Player_Hash = C.GetPlayerIDHash();
        
        For (Inv = C.Pawn.Inventory; Inv != None; Inv = Inv.Inventory )
                {
                If (Weapon(Inv) != None) // Fill in the blanks.
                        {
                        k = Player_Info[i].Player_Weapons.Length;
			Player_Info[i].Player_Weapons.Insert(k, 1);
                        Player_Info[i].Player_Weapons[k].WeaponClass = String(Inv.Class);
                        Player_Info[i].Player_Weapons[k].Primary_Ammo = Weapon(Inv).AmmoCharge[0];
                        Player_Info[i].Player_Weapons[k].Secondary_Ammo = Weapon(Inv).AmmoCharge[1];
                        }
                }
        // as many of these as you need. Also modify the struct

        Player_Info[i].Player_PRI.LightSuitPower = SPPRI(C.PlayerReplicationInfo).LightSuitPower;
        Player_Info[i].Player_PRI.BlinderPower = SPPRI(C.PlayerReplicationInfo).BlinderPower;
        Player_Info[i].Player_PRI.StealthPower = SPPRI(C.PlayerReplicationInfo).StealthPower;
        Player_Info[i].Player_PRI.Credits = SPPRI(C.PlayerReplicationInfo).Credits;
        
        SaveConfig();
}

Function LoadPlayerInfo(PlayerController C)
{
        Local Int i, k;
        Local Weapon W;
        Local Inventory Inv;

	Player_Info.Insert(i, 1);

        For (i = 0; i < Player_Info.Length; i++)
                {
                If (Player_Info[i].Player_hash == C.GetPlayerIDHash())
                        {
                        Break;
                        }
                }

	Player_Info[i].Player_Weapons.Insert(k, 1);
        
        For (k = 0; k < Player_Info[i].Player_Weapons.Length; k++) // fix - setprimaryammoto to the right variable name
        {
                W = Spawn(Class<Weapon>(DynamicLoadObject(Player_Info[i].Player_Weapons[k].WeaponClass, Class'Class')));
		if (W != None && C.Pawn != None)
		{
                	W.GiveTo(C.Pawn);
                
                	W.AmmoCharge[0] = Player_Info[i].Player_Weapons[k].Primary_Ammo;
                	W.AmmoCharge[1] = Player_Info[i].Player_Weapons[k].Secondary_Ammo;
		}

        }
        
        For (k = 0; k < Player_Info[i].Player_Inventory.Length; k++)
                {
                Inv = Spawn(Class<Weapon>(DynamicLoadObject(Player_Info[i].Player_Inventory[k].Inventory_Class, Class'Class')));
		if (Inv != None && C.Pawn != None)
                	Inv.GiveTo(C.Pawn);
                }
        
        SPPRI(C.PlayerReplicationInfo).LightSuitPower = Player_Info[i].Player_PRI.LightSuitPower;
        SPPRI(C.PlayerReplicationInfo).BlinderPower = Player_Info[i].Player_PRI.BlinderPower;
        SPPRI(C.PlayerReplicationInfo).StealthPower = Player_Info[i].Player_PRI.StealthPower;
        SPPRI(C.PlayerReplicationInfo).Credits = Player_Info[i].Player_PRI.Credits;

	//log("HELLO PARSER. LOADPLAYERINFO WAS CALLED. KTHNXBYE.");
}

function RestartPlayer(Controller aPlayer)
{
	local SPLevelInfo O;

	Super.RestartPlayer(aPlayer);

	foreach AllActors(class'SPLevelInfo', O)
	{
		if (O != None)
		{
			bTransition = O.bIsATransition;
		}
	}

	if (bTransition)
	{
		if (PlayerController(aPlayer) != None)
			LoadPlayerInfo(PlayerController(aPlayer));
	}
}

defaultproperties
{
     	LoginMenuClass="GUI2K4.UT2K4PlayerLoginMenu"
	DefaultPlayerClassName="CodeBase.SPPawn"
	Description="Single Player Framework to use either as a template for a full SP mod or for a series of maps."
	GameName="Single Player"
        MapListType="CodeBase.SPMapList"
        MutatorClass="CodeBase.SPMutator"
	TimeLimit=0
	GoalScore=0
	bAllowTrans=false
	Acronym="SP"
	MapPrefix="SP"
	//EndGameSound[0]=None
	//EndGameSound[1]=None
	//AltEndGameSound[0]=None
	//AltEndGameSound[1]=None
     	PlayerControllerClassName="CodeBase.SPPlayer"
     	HUDType="CodeBase.SPHUD"
	bAllowVehicles=true
	bQuickStart=true
}